#ifndef FunctionsLib_h
#define FunctionsLib_h

// Debugging functions
void DebugPrintln(int type, const char* x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, char x[]){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, float x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, char x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, int x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, uint32_t x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, unsigned long x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, String x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrintln(int type, IPAddress x){
  if(type <= debug){
    Serial.println(x);
  }
}

void DebugPrint(int type, const char* x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, char x[]){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, float x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, char x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, int x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, uint32_t x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, unsigned long x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, String x){
  if(type <= debug){
    Serial.print(x);
  }
}

void DebugPrint(int type, IPAddress x){
  if(type <= debug){
    Serial.print(x);
  }
}


// Interrupt Service Routine (Increment counter)
void ICACHE_RAM_ATTR interruptRoutine1() {
  counter[0]++;
}
void ICACHE_RAM_ATTR interruptRoutine2() {
  counter[1]++;
}
void ICACHE_RAM_ATTR interruptRoutine3() {
  counter[2]++;
}
void ICACHE_RAM_ATTR interruptRoutine4() {
  counter[3]++;
}

// Calculating all values
void CalcValues()
{
  result[0] = counter[0] * 60;      // Engine speed [rpm]
  result[1] = counter[1] * 60;      // Shaft speed [rpm]
  result[2] = counter[2] * 0.13477; // Sea water flow [l/min] (slope = 10/74.2 = 7,42)
  result[3] = counter[3] * 0.13477; // Sea water flow [l/min] (slope = 10/74.2 = 7,42)
  for (int i = 0; i < 4; i++)     // For counter 0 and 1
  {
    counter[i] = 0;               // Clear counter
  }
}

// Calculating live time (loadEEPROM and saveEEPROM only in mail loop possible and not in subrutines)
void LiveTime(){
  if(result[0] > 0){       // If engine on then counting the live time
    saveLiveTime = 1;      // set the marker for saving (saveEEPROM is only possible in the main loop)
  }
  else{
    saveLiveTime = 0;      // clear the marker for non saving
  }
}

// Checksum calculation for NMEA
char CheckSum(String NMEAData) {
  char checksum = 0;
  // Iterate over the string, XOR each byte with the total sum
  for (int c = 0; c < NMEAData.length(); c++) {
    checksum = char(checksum ^ NMEAData.charAt(c));
  } 
  // Return the result
  return checksum;
}

void eraseEEPROMConfig(configData cfg) {
  // Reset EEPROM bytes to '0' for the length of the data structure
  EEPROM.begin(sizeEEPROM);
  for (int i = cfgStart ; i < sizeof(cfg) ; i++) {
    EEPROM.write(i, 0);
  }
  delay(200);
  EEPROM.commit();
  EEPROM.end();
}

void saveEEPROMConfig(configData cfg) {
  // Save configuration from RAM into EEPROM
  EEPROM.begin(sizeEEPROM);
  EEPROM.put(cfgStart, cfg);
  delay(200);
  EEPROM.commit();                      // Only needed for ESP8266 to get data written
  EEPROM.end();                         // Free RAM copy of structure
}

configData loadEEPROMConfig() {
  // Loads configuration from EEPROM into RAM
  configData cfg;
  EEPROM.begin(sizeEEPROM);
  EEPROM.get(cfgStart, cfg);
  EEPROM.end();
  return cfg;
}

void saveEEPROMLiveTime(float livetime) {
  // Save configuration from RAM into EEPROM
  EEPROM.begin(sizeEEPROM);
  EEPROM.put(ltcStart, livetime);
  delay(200);
  EEPROM.commit();                      // Only needed for ESP8266 to get data written
  EEPROM.end();                         // Free RAM copy of structure
}

float loadEEPROMLiveTime() {
  // Loads configuration from EEPROM into RAM
  float livetime;
  EEPROM.begin(sizeEEPROM);
  EEPROM.get(ltcStart, livetime);
  EEPROM.end();
  return livetime;
}

 // Converting string to float
 float toFloat(String settingValue){
 char floatbuf[settingValue.length()+1];
 settingValue.toCharArray(floatbuf, sizeof(floatbuf));
 float f = atof(floatbuf);
 return f;
 }
 
 // Converting string to long
 long toLong(String settingValue){
 char longbuf[settingValue.length()+1];
 settingValue.toCharArray(longbuf, sizeof(longbuf));
 long l = atol(longbuf);
 return l;
 }
 
 // Converting String to integer and then to boolean
 // 1 = true
 // 0 = false
 boolean toBoolean(String settingValue) {
 if(settingValue.toInt()==1){
 return true;
 } else {
 return false;
 }
 }

 /* Apply the value to the parameter by searching for the parameter name
 Using String.toInt(); for Integers
 toFloat(string); for Float
 toBoolean(string); for Boolean
 toLong(string); for Long
 */
 void applySetting(String settingName, String settingValue) {
 
 int exINT;
 float exFloat;
 boolean exBoolean;
 long exLong;
 char exChar[31]; 
 
 if(settingName == "valid") {
 exINT=settingValue.toInt();
 }
 
 if(settingName == "exFloat") {
 exFloat=toFloat(settingValue);
 }
 
 if(settingName == "exBoolean") {
 exBoolean=toBoolean(settingValue);
 }
 
 if(settingName == "exLong") {
 exLong=toLong(settingValue);
 }

 // String to Char
 if(settingName == "exString") {
 settingValue.toCharArray(exChar, 31);
 }
 
 }

// Read configuration data from SDCard
configData readSDConfig(char* configfile){
 char character;
 String settingName;
 String settingValue;
 configData conf;
 File myFile;
 myFile = SD.open(configfile);
 if (myFile) {
   while (myFile.available()) {
   character = myFile.read();
   while((myFile.available()) && (character != '[')){
   character = myFile.read();
   }
   character = myFile.read();
   while((myFile.available()) && (character != '=')){
   settingName = settingName + character;
   character = myFile.read();
   }
   character = myFile.read();
   while((myFile.available()) && (character != ']')){
   settingValue = settingValue + character;
   character = myFile.read();
   }
     if(character == ']'){
 
       //Debuuging Printing
       DebugPrint(3, settingName);
       DebugPrint(3, ": ");
       DebugPrintln(3, settingValue);
 
       // Apply the value to the parameter
       if(settingName == "valid") {
         conf.valid = settingValue.toInt();
       }
       if(settingName == "debug") {
         conf.debug = settingValue.toInt();
       }
       if(settingName == "ssid") {
         settingValue.toCharArray(conf.ssid, 31);
       }
       if(settingName == "password") {
         settingValue.toCharArray(conf.password, 31);
       }
       if(settingName == "serverMode") {
         settingValue.toCharArray(conf.serverMode, 10);
       }
       if(settingName == "serspeed") {
         conf.serspeed = settingValue.toInt();
       }
       if(settingName == "engineID") {
         conf.engineID = settingValue.toInt();
       }
       if(settingName == "speedSensorType") {
         settingValue.toCharArray(conf.speedSensorType, 10);
       }
       if(settingName == "engineSpeed") {
         conf.engineSpeed = settingValue.toInt();
       }
       if(settingName == "shaftSpeed") {
         conf.shaftSpeed = settingValue.toInt();
       }
       if(settingName == "offsetLiveTimeCounter") {
         conf.offsetLiveTimeCounter = settingValue.toInt();
       }
       if(settingName == "liveTimeCounter") {
         conf.liveTimeCounter = settingValue.toInt();
       }
       if(settingName == "flowSensorType") {
         settingValue.toCharArray(conf.flowSensorType, 10);
       }
       if(settingName == "seaWaterFlow") {
         conf.seaWaterFlow = settingValue.toInt();
       }
       if(settingName == "coolingWaterFlow") {
         conf.coolingWaterFlow = settingValue.toInt();
       }
       if(settingName == "tempSensorType") {
         settingValue.toCharArray(conf.tempSensorType, 10);
       }
       if(settingName == "tempUnit") {
         settingValue.toCharArray(conf.tempUnit, 2);
       }
       if(settingName == "seaWaterTemp") {
         conf.seaWaterTemp = settingValue.toInt();
       }
       if(settingName == "seaWaterTempID") {
         conf.seaWaterTempID = settingValue.toInt();
       }
       if(settingName == "coolingWaterTemp") {
         conf.coolingWaterTemp = settingValue.toInt();
       }
       if(settingName == "coolingWaterTempID") {
         conf.coolingWaterTempID = settingValue.toInt();
       }
       if(settingName == "engineRoomTemp") {
         conf.engineRoomTemp = settingValue.toInt();
       }
       if(settingName == "engineRoomTempID") {
         conf.engineRoomTempID = settingValue.toInt();
       }
       if(settingName == "safetyRelay") {
         conf.safetyRelay = settingValue.toInt();
       }
       if(settingName == "roomSensorType") {
         settingValue.toCharArray(conf.roomSensorType, 10);
       }
       if(settingName == "roomTemp") {
         conf.roomTemp = settingValue.toInt();
       }
       if(settingName == "roomHumidity") {
         conf.roomHumidity = settingValue.toInt();
       }
 
       // Reset Strings
       settingName = "";
       settingValue = "";
     }
   }
   // close the file:
   myFile.close();
   DebugPrintln(3, "Reading done");
   return conf;
 }
 else {
   // if the file didn't open, print an error:
   DebugPrint(1, "Error opening ");
   DebugPrintln(1, configfile);
 }
}
 
 // Writes a configuration file to SDCard
 void writeSDConfig(char* configfile) {
  
 int exINT = 15;
 float exFloat = 1.12345;
 boolean exBoolean = true;
 long exLong = 2123456789; 
 
 File myFile;
 
 // Delete the old One
 SD.remove(configfile);
 // Create new one
 myFile = SD.open(configfile, FILE_WRITE);
 // Writing in the file
 if (myFile) {
  
   configData defconf;// Definition of default configuration data

   myFile.println("# Configuration file for NMEA server");
   myFile.println("# Norbert Walter (C) 2018");
   myFile.println("");
   
   myFile.print("[");
   myFile.print("valid=");
   myFile.print(defconf.valid);
   myFile.println("]");
   
   myFile.print("[");
   myFile.print("debug=");
   myFile.print(defconf.debug);
   myFile.println("]");
   
   myFile.print("[");
   myFile.print("ssid=");
   myFile.print(defconf.ssid);
   myFile.println("]");
   
   myFile.print("[");
   myFile.print("password=");
   myFile.print(defconf.password);
   myFile.println("]");
   
   myFile.print("[");
   myFile.print("serverMode=");
   myFile.print(defconf.serverMode);
   myFile.println("]");
   
   myFile.print("[");
   myFile.print("serspeed=");
   myFile.print(defconf.serspeed);
   myFile.println("]");

   myFile.print("[");
   myFile.print("engineID=");
   myFile.print(defconf.engineID);
   myFile.println("]");

   myFile.print("[");
   myFile.print("speedSensorType=");
   myFile.print(defconf.speedSensorType);
   myFile.println("]");

   myFile.print("[");
   myFile.print("engineSpeed=");
   myFile.print(defconf.engineSpeed);
   myFile.println("]");

   myFile.print("[");
   myFile.print("shaftSpeed=");
   myFile.print(defconf.shaftSpeed);
   myFile.println("]");

   myFile.print("[");
   myFile.print("offsetLiveTimeCounter=");
   myFile.print(defconf.offsetLiveTimeCounter);
   myFile.println("]");

   myFile.print("[");
   myFile.print("liveTimeCounter=");
   myFile.print(defconf.liveTimeCounter);
   myFile.println("]");

   myFile.print("[");
   myFile.print("flowSensorType=");
   myFile.print(defconf.flowSensorType);
   myFile.println("]");

   myFile.print("[");
   myFile.print("seaWaterFlow=");
   myFile.print(defconf.seaWaterFlow);
   myFile.println("]");

   myFile.print("[");
   myFile.print("coolingWaterFlow=");
   myFile.print(defconf.coolingWaterFlow);
   myFile.println("]");

   myFile.print("[");
   myFile.print("tempSensorType=");
   myFile.print(defconf.tempSensorType);
   myFile.println("]");

   myFile.print("[");
   myFile.print("tempUnit=");
   myFile.print(defconf.tempUnit);
   myFile.println("]");

   myFile.print("[");
   myFile.print("seaWaterTemp=");
   myFile.print(defconf.seaWaterTemp);
   myFile.println("]");

   myFile.print("[");
   myFile.print("seaWaterTempID=");
   myFile.print(defconf.seaWaterTempID);
   myFile.println("]");

   myFile.print("[");
   myFile.print("coolingWaterTemp=");
   myFile.print(defconf.coolingWaterTemp);
   myFile.println("]");

   myFile.print("[");
   myFile.print("coolingWaterTempID=");
   myFile.print(defconf.coolingWaterTempID);
   myFile.println("]");

   myFile.print("[");
   myFile.print("engineRoomTemp=");
   myFile.print(defconf.engineRoomTemp);
   myFile.println("]");

   myFile.print("[");
   myFile.print("engineRoomTempID=");
   myFile.print(defconf.engineRoomTempID);
   myFile.println("]");

   myFile.print("[");
   myFile.print("safetyRelay=");
   myFile.print(defconf.safetyRelay);
   myFile.println("]");

   myFile.print("[");
   myFile.print("roomSensorType=");
   myFile.print(defconf.roomSensorType);
   myFile.println("]");

   myFile.print("[");
   myFile.print("roomTemp=");
   myFile.print(defconf.roomTemp);
   myFile.println("]");

   myFile.print("[");
   myFile.print("roomHumidity=");
   myFile.print(defconf.roomHumidity);
   myFile.println("]");
   
   // close the file:
   myFile.close();
   DebugPrintln(3, "Writing done");
 }
 else{
 // If the file didn't open, print an error:
 DebugPrint(1, "Error opening ");
 DebugPrintln(1, configfile);
 }
 }

#endif
